@@ -18,13 +18,15 @@ This is to avoid maintaining two nearly-identical code-paths.  Running only one
 policy is obviously a special case of running many.
 
 """
-from chapps.config import config  # the global instance of the config object
+# from chapps.config import config  # the global instance of the config object
 from chapps.policy import (
+    EmailPolicy,
     OutboundQuotaPolicy,
     GreylistingPolicy,
     SenderDomainAuthPolicy,
 )
-from chapps.util import AttrDict, PostfixPolicyRequest
+from chapps.util import PostfixPolicyRequest
+from chapps.inbound import InboundPPR
 from chapps.outbound import OutboundPPR
 from chapps.signals import (
     CHAPPSException,
@@ -33,13 +35,13 @@ from chapps.signals import (
     AuthenticationFailureException,
 )
 from functools import cached_property
-from typing import Type
+from typing import Type, Optional, List
 import logging
 import asyncio
 
 try:
     from chapps.spf_policy import SPFEnforcementPolicy
-except:
+except Exception:
     HAVE_SPF = False
     pass
 else:
@@ -48,132 +50,6 @@ else:
 logger = logging.getLogger(__name__)  # pragma: no cover
 
 
-class RequestHandler:
-    """Deprecated base class for wrapping policy managers in an event loop
-
-    This workhorse base class generalizes creating an event loop around an
-    email policy.  It has been superseded by :class:`.CascadingPolicyHandler`,
-    which expands on the logic of this (original) class by allowing any number
-    of policies to be applied in order.
-
-    Instance attributes:
-
-      :policy: the :class:`~chapps.policy.EmailPolicy` manager instance
-
-      :config: a reference to the :class:`~chapps.config.CHAPPSConfig` stored
-        on the policy instance.
-
-      :pprclass: a reference to the particular kind of
-        :class:`~chapps.util.PostfixPolicyRequest` to instantiate
-
-    """
-
-    def __init__(
-        self,
-        policy,
-        *,
-        pprclass: Type[PostfixPolicyRequest] = PostfixPolicyRequest,
-    ):
-        """Setup a Postfix policy request handler
-
-        :param chapps.policy.EmailPolicy policy: an instance of a policy
-          manager (a subclass of :class:`~chapps.policy.EmailPolicy`)
-
-        :param Type[PostfixPolicyRequest] pprclass: the subclass of :class:`~chapps.util.PostfixPolicyRequest` to instantiate from the Postfix payloads; defaults to :class:`~chapps.util.PostfixPolicyRequest`
-
-        .. note::
-
-          Unlike other class families within CHAPPS, the handlers in this
-          module do not accept config-override arguments.  They obtain their
-          references to the config from their attached policy managers.
-
-        """
-        self.policy = policy
-        self.config = self.policy.config  # in case a custom config is in use
-        self.pprclass = pprclass
-
-    @cached_property
-    def listen_address(self):
-        return self.policy.params.listen_address
-
-    @cached_property
-    def listen_port(self):
-        return self.policy.params.listen_port
-
-    def async_policy_handler(self):
-        """Coroutine factory
-
-        :returns: a coroutine which handles requests by Postfix to the policy
-
-        The policy handler closure is meant to do all the weird grunt work
-        associated with talking to some other application over a socket.  It
-        listens, and it minimally processes the payload it receives in order to
-        pass something sane to the rest of the library.
-
-        Part of this is accomplished through use of the utility class
-        :class:`~chapps.util.PostfixPolicyRequest` and its subclass(es).  The
-        utility class turns the payload into something that the rest of the
-        code can work with easily.
-
-        """
-        pprclass = self.pprclass
-        policy = self.policy
-        accept = policy.params.acceptance_message
-        reject = policy.params.rejection_message
-        encoding = config.chapps.payload_encoding
-        logger.debug(
-            f"Policy handler requested for {type(policy).__name__}"
-            f" using PPR class {pprclass.__name__}."
-        )
-
-        async def handle_policy_request(reader, writer) -> None:
-            """Handles reading and writing the streams around policy messages"""
-            while True:
-                try:
-                    policy_payload = await reader.readuntil(b"\n\n")
-                except ConnectionResetError:
-                    logger.debug(
-                        "Postfix said goodbye. Terminating this thread."
-                    )
-                    return
-                except asyncio.IncompleteReadError as e:
-                    logger.debug(
-                        "Postfix hung up before a read could be completed. Terminating this thread."
-                    )
-                    return
-                except CallableExhausted as e:
-                    raise e
-                except Exception:
-                    logger.exception("UNEXPECTED ")
-                    if reader.at_eof():
-                        logger.debug(
-                            "Postfix said goodbye oddly. Terminating this thread."
-                        )
-                        return
-                    continue
-                logger.debug(
-                    f"Payload received: {policy_payload.decode('utf-8')}"
-                )
-                policy_data = pprclass(
-                    policy_payload.decode(encoding).split("\n")
-                )
-                if policy.approve_policy_request(policy_data):
-                    resp = ("action=" + accept + "\n\n").encode()
-                    logger.debug(f"  .. Accepted.  Sending {resp}")
-                else:
-                    resp = ("action=" + reject + "\n\n").encode()
-                    logger.debug(f"  .. Rejected with {resp}")
-                try:
-                    writer.write(resp)
-                except asyncio.CancelledError:  # pragma: no cover
-                    pass
-                except Exception:
-                    logger.exception(f"Exception raised trying to send {resp}")
-                    return
-
-        return handle_policy_request
-
-
 class CascadingPolicyHandler:
     """Second-generation handler class which cascades multiple yes/no policies
 
@@ -203,7 +79,12 @@ class CascadingPolicyHandler:
 
     """
 
-    def __init__(self, policies=[], *, pprclass=PostfixPolicyRequest):
+    def __init__(
+        self,
+        policies: Optional[List[EmailPolicy]] = [],
+        *,
+        pprclass: PostfixPolicyRequest = PostfixPolicyRequest,
+    ):
         self.policies = policies
         self.pprclass = pprclass
         if not self.policies:
@@ -211,6 +92,12 @@ class CascadingPolicyHandler:
         self.config = self.policies[
             0
         ].config  # all copies of the config are the same
+        logger.debug(
+            "Grabbing config from file "
+            + self.config.chapps.config_file
+            + " via "
+            + self.policies[0].__class__.__name__
+        )
 
     @cached_property
     def listen_address(self):
@@ -238,6 +125,7 @@ class CascadingPolicyHandler:
         pprclass = self.pprclass
         policies = self.policies
         encoding = self.config.chapps.payload_encoding
+        no_user_key_response = self.config.chapps.no_user_key_response
         logger.debug(
             f"Cascading policy handler requested for "
             f"{[ type(p) for p in policies ]} using PPR "
@@ -319,7 +207,7 @@ class CascadingPolicyHandler:
                                 f"{type(policy).__name__} FAIL NS {policy_data}"
                             )
                     except AuthenticationFailureException:
-                        resp = "action=" + config.no_user_key_response + "\n\n"
+                        resp = "action=" + no_user_key_response + "\n\n"
                         approval = False
                         logger.info(
                             f"{type(policy).__name__} FAIL NA {policy_data}"
@@ -341,6 +229,60 @@ class CascadingPolicyHandler:
         return handle_policy_request
 
 
+class RequestHandler(CascadingPolicyHandler):
+    """Refactored intermediate base class for wrapping policy managers in an event loop
+
+    This class has been reimplemented as a subclass of
+    :class:`.CascadingPolicyHandler`, as a special case which has only one
+    policy to handle.
+
+    Instance attributes:
+
+      :policy: the :class:`~chapps.policy.EmailPolicy` manager instance
+
+      :config: a reference to the :class:`~chapps.config.CHAPPSConfig` stored
+        on the policy instance.
+
+      :pprclass: a reference to the particular kind of
+        :class:`~chapps.util.PostfixPolicyRequest` to instantiate
+
+    """
+
+    def __init__(
+        self,
+        policy: EmailPolicy,
+        *,
+        pprclass: Type[PostfixPolicyRequest] = PostfixPolicyRequest,
+    ):
+        """Setup a Postfix policy request handler
+
+        :param chapps.policy.EmailPolicy policy: an instance of a policy
+          manager (a subclass of :class:`~chapps.policy.EmailPolicy`)
+
+        :param Type[PostfixPolicyRequest] pprclass: the subclass of :class:`~chapps.util.PostfixPolicyRequest` to instantiate from the Postfix payloads; defaults to :class:`~chapps.util.PostfixPolicyRequest`
+
+        .. note::
+
+          Unlike other class families within CHAPPS, the handlers in this
+          module do not accept config-override arguments.  They obtain their
+          references to the config from their attached policy managers.
+
+        """
+        super().__init__([policy], pprclass=pprclass)
+
+    @cached_property
+    def listen_address(self):
+        return self.policy.params.listen_address
+
+    @cached_property
+    def listen_port(self):
+        return self.policy.params.listen_port
+
+    @cached_property
+    def policy(self):
+        return self.policies[0]
+
+
 class OutboundMultipolicyHandler(CascadingPolicyHandler):
     """Convenience subclass for combining outbound P/F policies
 
@@ -397,7 +339,7 @@ class GreylistingHandler(RequestHandler):
 
         """
         p = policy or GreylistingPolicy()
-        super().__init__(p)
+        super().__init__(p, pprclass=InboundPPR)
 
 
 class SenderDomainAuthHandler(RequestHandler):
@@ -414,9 +356,147 @@ class SenderDomainAuthHandler(RequestHandler):
         super().__init__(p, pprclass=OutboundPPR)
 
 
+class CascadingMultiresultPolicyHandler(CascadingPolicyHandler):
+    """Cascading handler for policies which produce more than two results
+
+    Some policies, such as the SPF enforcement policy, need to be able to
+    generate responses with more nuance than simply pass/fail, and so there is
+    a need for a handler which can deal with policies which return strings
+    (Postfix directives), or possibly a custom class to encapsulate also some
+    idea of pass/fail, in order to know whether to abort the policy-evaluation
+    loop early.
+
+    """
+
+    def async_policy_handler(self):
+        """Returns a coroutine which handles results according to the configuration
+
+        The policy being enforced is stored in the SPF-related TXT record
+        on the sender's domain.  The local configuration of this policy
+        amounts to instructions about responses to different outcomes of
+        the SPF check, along with what IP address and port to listen on.
+
+        This policy handler is different from others in that, because it
+        does not expect a PASS/FAIL response, it simply wraps the return
+        value of
+        :func:`~chapps.spf_policy.SPFEnforcementPolicy.approve_policy_request()`
+        in a Postfix response packet, and sends it.  Rather than refer to
+        pre-configured acceptance and rejection messages, it expects the
+        approval routine to send a string which can be interpreted by
+        Postfix as a command.
+
+        TODO: In order to be able to cascade *through* this kind of policy,
+        it is going to have to return a first-class object which can be
+        annotated as being a pass or a fail, so that a cascading handler
+        can decide whether to continue.  That object's `__str__()`
+        method will need to return the Postfix command.
+
+        .. todo::
+
+          In order to support more than one MTA (tho no such support is
+          planned), the action-translation layer might be refactored out of
+          the policy itself, to be applied here, in order to switch between
+          different types.
+
+        """
+        ### This override version for SPF enforcement does not assume a yes-or-no response pattern
+        logger.debug(
+            "Policy handler requested for"
+            f" {[type(p).__name__ for p in self.policies]!r}."
+        )
+        pprclass = self.pprclass
+        policies = self.policies
+        encoding = self.config.chapps.payload_encoding
+
+        logger.debug(
+            "Cascading Multipolicy handler requested for "
+            f"{[type(p) for p in policies]} using PPR "
+            f"class {pprclass.__name__}."
+        )
+
+        async def handle_policy_request(reader, writer):
+            """Handles reading and writing the streams around policy approval messages"""
+            while True:
+                try:
+                    policy_payload = await reader.readuntil(b"\n\n")
+                except ConnectionResetError:
+                    logger.debug(
+                        "Postfix said goodbye. Terminating this thread."
+                    )
+                    return
+                except asyncio.IncompleteReadError as e:
+                    logger.debug(
+                        "Postfix hung up before a read could be completed."
+                        " Terminating this thread."
+                    )
+                    return
+                except CallableExhausted as e:
+                    raise e
+                except Exception:
+                    if reader.at_eof():
+                        logger.debug(
+                            "Postfix said goodbye oddly."
+                            " Terminating this thread."
+                        )
+                        return
+                    else:
+                        logger.exception("UNEXPECTED ")
+                    continue
+                logger.debug(
+                    f"Payload received: {policy_payload.decode(encoding)}"
+                )
+                policy_data = pprclass(
+                    policy_payload.decode(encoding).split("\n")
+                )
+
+                actions = ["DUNNO"]
+                # TODO:
+                # track all responses, and then extract non-DUNNO ones if any
+                for policy in policies:
+                    try:
+                        action = policy.approve_policy_request(policy_data)
+                        actions.append(action)
+                        logger.info(
+                            f"{type(policy).__name__} "
+                            # + ("PASS" if action else "FAIL")
+                            + f" {action!r} {policy_data}"
+                        )
+                    except CHAPPSException:
+                        logger.exception("During policy evaluation:")
+                    if not action:
+                        break
+
+                if action:  # i.e. the mail will be forwarded
+                    non_dunno = [a for a in actions if a != "DUNNO"]
+                    action = non_dunno[-1] if non_dunno else "DUNNO"
+                try:
+                    writer.write((f"action={action}\n\n").encode(encoding))
+                except asyncio.CancelledError:  # pragma: no cover
+                    pass
+                except Exception:
+                    logger.exception(
+                        f"Exception raised trying to send {action}"
+                    )
+                    return
+
+        return handle_policy_request
+
+
+class MultiresultPolicyHandler(CascadingMultiresultPolicyHandler):
+    """A subclass for handling just one policy"""
+
+    def __init__(
+        self,
+        policy: EmailPolicy,
+        *,
+        pprclass: Optional[PostfixPolicyRequest] = PostfixPolicyRequest,
+    ):
+        super().__init__([policy], pprclass=pprclass)
+
+
 if HAVE_SPF:
 
-    class SPFEnforcementHandler(RequestHandler):
+    class SPFEnforcementHandler(MultiresultPolicyHandler):
         """Special handler class for :class:`~chapps.spf_policy.SPFEnforcementPolicy`
 
         This one came along last and forced a reconsideration of how all this
@@ -433,7 +513,11 @@ if HAVE_SPF:
 
         """
 
-        def __init__(self, policy: SPFEnforcementPolicy = None):
+        def __init__(
+            self,
+            policy: Optional[SPFEnforcementPolicy] = None,
+            pprclass: Optional[PostfixPolicyRequest] = InboundPPR,
+        ):
             """Set up an SPFEnforcementHandler
 
             :param chapps.spf_policy.SPFEnforcementPolicy policy: an instance
@@ -441,83 +525,32 @@ if HAVE_SPF:
 
             """
             p = policy or SPFEnforcementPolicy()
-            super().__init__(p)
+            super().__init__(p, pprclass=pprclass)
 
-        def async_policy_handler(self):
-            """Returns a coroutine which handles results according to the configuration
+        @cached_property
+        def policy(self):
+            return self.policies[0]
 
-            The policy being enforced is stored in the SPF-related TXT record
-            on the sender's domain.  The local configuration of this policy
-            amounts to instructions about responses to different outcomes of
-            the SPF check, along with what IP address and port to listen on.
+    class InboundMultipolicyHandler(CascadingMultiresultPolicyHandler):
+        """Implements SPF and Greylisting simultaneously
 
-            This policy handler is different from others in that, because it
-            does not expect a PASS/FAIL response, it simply wraps the return
-            value of
-            :func:`~chapps.spf_policy.SPFEnforcementPolicy.approve_policy_request()`
-            in a Postfix response packet, and sends it.  Rather than refer to
-            pre-configured acceptance and rejection messages, it expects the
-            approval routine to send a string which can be interpreted by
-            Postfix as a command.
+        This is a template for an inbound multipolicy handler, which by default
+        checks SPF and also performs Greylisting, each of which depends upon
+        options which the email account administrator may set about whether
+        either of SPF or Greylisting or both should be applied to inbound mail
+        for the particular domain.
 
-            TODO: In order to be able to cascade *through* this kind of policy,
-            it is going to have to return a first-class object which can be
-            annotated as being a pass or a fail, so that a cascading handler
-            can decide whether to continue.  That object's `__str__()`
-            method will need to return the Postfix command.
+        """
 
-            .. todo::
-
-              In order to support more than one MTA (tho no such support is
-              planned), the action-translation layer might be refactored out of
-              the policy itself, to be applied here, in order to switch between
-              different types.
-
-            """
-            ### This override version for SPF enforcement does not assume a yes-or-no response pattern
-            logger.debug(
-                f"Policy handler requested for {type(self.policy).__name__}."
-            )
-            policy = self.policy
-            encoding = config.chapps.payload_encoding
-
-            async def handle_policy_request(reader, writer):
-                """Handles reading and writing the streams around policy approval messages"""
-                while True:
-                    try:
-                        policy_payload = await reader.readuntil(b"\n\n")
-                    except ConnectionResetError:
-                        logger.debug(
-                            "Postfix said goodbye. Terminating this thread."
-                        )
-                        return
-                    except CallableExhausted as e:
-                        raise e
-                    except Exception:
-                        logger.exception("UNEXPECTED ")
-                        if reader.at_eof():
-                            logger.debug(
-                                "Postfix said goodbye oddly. Terminating this thread."
-                            )
-                            return
-                        continue
-                    logger.debug(
-                        f"Payload received: {policy_payload.decode(encoding)}"
-                    )
-                    policy_data = PostfixPolicyRequest(
-                        policy_payload.decode(encoding).split("\n")
-                    )
-                    action = policy.approve_policy_request(policy_data)
-                    resp = ("action=" + action + "\n\n").encode()
-                    logger.debug(f"  .. SPF Enforcement sending {resp}")
-                    try:
-                        writer.write(resp)
-                    except asyncio.CancelledError:  # pragma: no cover
-                        pass
-                    except Exception:
-                        logger.exception(
-                            f"Exception raised trying to send {resp}"
-                        )
-                        return
-
-            return handle_policy_request
+        def __init__(
+            self,
+            policies: Optional[List[EmailPolicy]] = None,
+            *,
+            pprclass: Optional[PostfixPolicyRequest] = InboundPPR,
+        ):
+            """Create an inbound policy handler for SPF + Greylisting"""
+            policies = policies or [
+                SPFEnforcementPolicy(),
+                GreylistingPolicy(),
+            ]
+            super().__init__(policies, pprclass=pprclass)
